#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/timeb.h>
#include <signal.h>
#include <time.h>
#include "ratednet.h"
#include "lib.h"

struct rated * r;
int verbose = 0;
int polltime = 1;
int bitunits = 0;
int human = 0;
int quiet = 0;
int nresults = 0;
int eqw = 0;

void help(unsigned char * name)
{
        printf("bmtools 0.7 - Mateusz 'mteg' Golicz <mtg@elsat.net.pl>, 2003\n", name);
	printf("usage: %s [options] (-l host:port | datasource@host[:port])\n", name);
	printf("  -h        Show this help\n");
	printf("  -r <sec>  Poll datasource every t seconds\n");
	printf("  -u        Human-friendly units\n");
	printf("  -m        Multiline (more readable) output. Implies -u.\n");
	printf("  -b        Bit units.\n");
	printf("  -q <n>    Quit after printing n results)\n");
	printf("  -x        No ASCII-graphs, only text\n");
	printf("  -v        Verbose (show connection information)\n");
	printf("  -l        List data sources on that host\n");
}


int main(int argc, char ** argv)
{
	int c;
	int listmode = 0;
	unsigned char * host;
	unsigned char * pmark;
	int port = 6000, n, i;
	u_int32_t ip;
	unsigned char * dsn;
	int dsid;
	struct dseinfo dse;
	struct dsdata dsd;
	unsigned long long startBytes = 0;
	unsigned long startPackets = 0;
	struct timeb start;
	
	while((c = getopt(argc, argv, "hlr:ubq:vxm")) != EOF)
	{
		switch(c)
		{
			case 'h': help(argv[0]); exit(0);
			case 'l': listmode = 1; break;
			case 'x': quiet = 1; break;
			case 'v': verbose = 1; break;
			case 'b': bitunits = 1; break;
			case 'u': human = 1; break;
			case 'm': eqw = human = 1; break;
			case 't': polltime = atoi(optarg); if(polltime < 1) polltime = 1; break;
			case 'q': nresults = atoi(optarg); if(nresults < 0) nresults = 0; break;
		}
	}
	
	if(!argv[optind])
	{
		help(argv[0]);
		exit(1);
	}

	host = strdup(argv[optind]);

	if(!listmode)
	{
		unsigned char * dsnmark;
		
		if(!(dsnmark = index(host, '@')))
		{
			help(argv[0]); exit(1);
		}
		*dsnmark = 0;
		dsn = host;
		host = dsnmark + 1;
	}
		
	if((pmark = index(host, ':')))
	{
		*pmark = 0;
		port = atoi(pmark + 1);
		if(port < 1 || port > 65535)
		{
			help(argv[0]);
			exit(1);
		}
	}
		
	if(!gethost(host, &ip))
	{
		fprintf(stderr, "%s: Host unknown\n", host);
		return 1;
	}
		
	n = rtn_count(ip, port, verbose);
	if(verbose) printf("Got response: %d data sources.\n", n);

	if(!n)
	{
		fprintf(stderr, "Timed out or protocol error.\n");
		return 1;
	}
	for(i = 0; i<n; i++)
	{
		if(rtn_dsinfo(ip, port, verbose, i, &dse))
		{
			if(listmode)
				printf("%2d: %-20s %s\n", i, dse.shortname, dse.description);
			else
				if(strcmp(dsn, dse.shortname) == 0) break;
		}
	}
	
	if(listmode) return 0;
	
	if(i == n)
	{
		fprintf(stderr, "No such data source: %s on %s.\n", dsn, host);
		exit(1);
	}
	dsid = i;
	
	while(1)
	{
		time_t stamp;
		
		if(rtn_poll(ip, port, verbose, dsid, &dsd, &stamp))
		{
			float cB, cP, aB, aP, oB, oP;
			unsigned char cBs[20], cPs[20], aBs[20], aPs[20], oBs[20], oPs[20];
			unsigned char * unit;
			
			cB = dsd.currentBytes;
			cP = dsd.currentPackets;
			
			if(!startBytes)
			{
				startBytes = dsd.overallBytes;
				startPackets = dsd.overallPackets;
				ftime(&start);
				aB = 0;
				aP = 0;
			}
			else
			{
				struct timeb now;
				unsigned long miliFromStart;
				ftime(&now);
				
				miliFromStart = (now.time - start.time) * 1000 + now.millitm - start.millitm;
				
				aB = dsd.overallBytes - startBytes;
				aP = dsd.overallPackets - startPackets;
				aB /= miliFromStart; aB *= 1000;
				aP /= miliFromStart; aP *= 1000;
				
			}
			
			if(stamp)
			{
				oB = dsd.overallBytes; oB /= stamp;
				oP = dsd.overallPackets; oP /= stamp;
			}
			else
			{
				oB = 0;
				oP = 0;
			}
			
			if(bitunits)
			{
				unit = "bps";
				oB *= 8; aB *= 8; cB *= 8;
			}
			else
				unit = "Bps";

			mkhuman(cB, 20, cBs, unit, 1024, human);
			mkhuman(aB, 20, aBs, unit, 1024, human);
			mkhuman(oB, 20, oBs, unit, 1024, human);
			mkhuman(cP, 20, cPs, "pps", 1000, human);
			mkhuman(aP, 20, aPs, "pps", 1000, human);
			mkhuman(oP, 20, oPs, "pps", 1000, human);

			if(eqw)
			{
				printf("=> Currently: %12s/%-11s\n", cBs, cPs);
				printf("   Average  : %12s/%-11s\n", aBs, aPs);
				printf("   Overall  : %12s/%-11s\n\n", oBs, oPs);
			}
			else
				printf("=> C: %s/%s A: %s/%s O: %s/%s\n", cBs, cPs, aBs, aPs, oBs, oPs);
			if((--nresults) == 0) break;
		}
		
		usleep(polltime * 1000000);
	}
	
	
	return(0);
}
